@@ -1,25 +1,16 @@
 /*
- * "$Id: testhttp.c 5679 2006-06-20 14:45:15Z mike $"
+ * "$Id: testhttp.c 10899 2013-03-11 18:44:36Z mike $"
  *
- *   HTTP test program for the Common UNIX Printing System (CUPS).
+ *   HTTP test program for CUPS.
  *
+ *   Copyright 2007-2011 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -32,10 +23,8 @@
  * Include necessary headers...
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include "http.h"
-#include "string.h"
+#include "string-private.h"
+#include "http-private.h"
 
 
 /*
@@ -84,6 +73,8 @@ static uri_test_t	uri_tests[] =	/* URI test data */
 			    "http", "username:password", "server", "/", 8080, 8080 },
 			  { HTTP_URI_OK, "http://username:passwor%64@server:8080/directory/filename",
 			    "http", "username:password", "server", "/directory/filename", 8080, 8080 },
+			  { HTTP_URI_OK, "http://[2000::10:100]:631/ipp",
+			    "http", "", "2000::10:100", "/ipp", 631, 631 },
 			  { HTTP_URI_OK, "https://username:passwor%64@server/directory/filename",
 			    "https", "username:password", "server", "/directory/filename", 443, 0 },
 			  { HTTP_URI_OK, "ipp://username:passwor%64@[::1]/ipp",
@@ -100,6 +91,10 @@ static uri_test_t	uri_tests[] =	/* URI test data */
 			    "ipp", "username:password", "fe80::200:1234:5678:9abc%eth0", "/ipp", 999, 999 },
 			  { HTTP_URI_OK, "http://server/admin?DEVICE_URI=usb://HP/Photosmart%25202600%2520series?serial=MY53OK70V10400",
 			    "http", "", "server", "/admin?DEVICE_URI=usb://HP/Photosmart%25202600%2520series?serial=MY53OK70V10400", 80, 0 },
+			  { HTTP_URI_OK, "lpd://Acme%20Laser%20(01%3A23%3A45).local._tcp._printer/",
+			    "lpd", "", "Acme Laser (01:23:45).local._tcp._printer", "/", 515, 0 },
+			  { HTTP_URI_OK, "ipp://HP%20Officejet%204500%20G510n-z%20%40%20Will's%20MacBook%20Pro%2015%22._ipp._tcp.local./",
+			    "ipp", "", "HP Officejet 4500 G510n-z @ Will's MacBook Pro 15\"._ipp._tcp.local.", "/", 631, 0 },
 
 			  /* Missing scheme */
 			  { HTTP_URI_MISSING_SCHEME, "/path/to/file/index.html",
@@ -136,6 +131,8 @@ static uri_test_t	uri_tests[] =	/* URI test data */
 			    "http", "", "", "", 80, 0 },
 			  { HTTP_URI_BAD_HOSTNAME, "http://serve%7/index.html",
 			    "http", "", "", "", 80, 0 },
+			  { HTTP_URI_BAD_HOSTNAME, "http://server with spaces/index.html",
+			    "http", "", "", "", 80, 0 },
 
 			  /* Bad port number */
 			  { HTTP_URI_BAD_PORT, "http://127.0.0.1:9999a/index.html",
@@ -143,7 +140,9 @@ static uri_test_t	uri_tests[] =	/* URI test data */
 
 			  /* Bad resource */
 			  { HTTP_URI_BAD_RESOURCE, "http://server/index.html%",
-			    "http", "", "server", "", 80, 0 }
+			    "http", "", "server", "", 80, 0 },
+ 			  { HTTP_URI_BAD_RESOURCE, "http://server/index with spaces.html",
+  			    "http", "", "server", "", 80, 0 }
 			};
 static const char * const base64_tests[][2] =
 			{
@@ -172,6 +171,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 {
   int		i, j, k;		/* Looping vars */
   http_t	*http;			/* HTTP connection */
+  http_encryption_t encryption;		/* Encryption type */
   http_status_t	status;			/* Status of GET command */
   int		failures;		/* Number of test failures */
   char		buffer[8192];		/* Input buffer */
@@ -252,7 +252,7 @@ main(int  argc,				/* I - Number of command-line arguments */
     for (i = 0, j = 0; i < (int)(sizeof(base64_tests) / sizeof(base64_tests[0])); i ++)
     {
       httpEncode64_2(encode, sizeof(encode), base64_tests[i][0],
-                     strlen(base64_tests[i][0]));
+                     (int)strlen(base64_tests[i][0]));
       decodelen = (int)sizeof(decode);
       httpDecode64_2(decode, &decodelen, base64_tests[i][1]);
 
@@ -306,7 +306,7 @@ main(int  argc,				/* I - Number of command-line arguments */
     * httpAddrGetList()
     */
 
-    fputs("httpAddrGetList(): ", stdout);
+    printf("httpAddrGetList(%s): ", hostname);
 
     addrlist = httpAddrGetList(hostname, AF_UNSPEC, NULL);
     if (addrlist)
@@ -328,6 +328,10 @@ main(int  argc,				/* I - Number of command-line arguments */
 
       httpAddrFreeList(addrlist);
     }
+    else if (isdigit(hostname[0] & 255))
+    {
+      puts("FAIL (ignored because hostname is numeric)");
+    }
     else
     {
       failures ++;
@@ -457,6 +461,66 @@ main(int  argc,				/* I - Number of command-line arguments */
 
     return (failures);
   }
+  else if (strstr(argv[1], "._tcp"))
+  {
+   /*
+    * Test resolving an mDNS name.
+    */
+
+    char	resolved[1024];		/* Resolved URI */
+
+
+    printf("_httpResolveURI(%s, _HTTP_RESOLVE_DEFAULT): ", argv[1]);
+    fflush(stdout);
+
+    if (!_httpResolveURI(argv[1], resolved, sizeof(resolved),
+                         _HTTP_RESOLVE_DEFAULT, NULL, NULL))
+    {
+      puts("FAIL");
+      return (1);
+    }
+    else
+      printf("PASS (%s)\n", resolved);
+
+    printf("_httpResolveURI(%s, _HTTP_RESOLVE_FQDN): ", argv[1]);
+    fflush(stdout);
+
+    if (!_httpResolveURI(argv[1], resolved, sizeof(resolved),
+                         _HTTP_RESOLVE_FQDN, NULL, NULL))
+    {
+      puts("FAIL");
+      return (1);
+    }
+    else if (strstr(resolved, ".local:"))
+    {
+      printf("FAIL (%s)\n", resolved);
+      return (1);
+    }
+    else
+    {
+      printf("PASS (%s)\n", resolved);
+      return (0);
+    }
+  }
+  else if (!strcmp(argv[1], "-u") && argc == 3)
+  {
+   /*
+    * Test URI separation...
+    */
+
+    uri_status = httpSeparateURI(HTTP_URI_CODING_ALL, argv[2], scheme,
+                                 sizeof(scheme), username, sizeof(username),
+				 hostname, sizeof(hostname), &port,
+				 resource, sizeof(resource));
+    printf("uri_status = %s\n", uri_status_strings[uri_status + 8]);
+    printf("scheme     = \"%s\"\n", scheme);
+    printf("username   = \"%s\"\n", username);
+    printf("hostname   = \"%s\"\n", hostname);
+    printf("port       = %d\n", port);
+    printf("resource   = \"%s\"\n", resource);
+
+    return (0);
+  }
 
  /*
   * Test HTTP GET requests...
@@ -482,7 +546,13 @@ main(int  argc,				/* I - Number of command-line arguments */
                     hostname, sizeof(hostname), &port,
 		    resource, sizeof(resource));
 
-    http = httpConnectEncrypt(hostname, port, HTTP_ENCRYPT_IF_REQUESTED);
+    if (!_cups_strcasecmp(scheme, "https") || !_cups_strcasecmp(scheme, "ipps") ||
+        port == 443)
+      encryption = HTTP_ENCRYPT_ALWAYS;
+    else
+      encryption = HTTP_ENCRYPT_IF_REQUESTED;
+
+    http = httpConnectEncrypt(hostname, port, encryption);
     if (http == NULL)
     {
       perror(hostname);
@@ -499,7 +569,6 @@ main(int  argc,				/* I - Number of command-line arguments */
     else
       printf("GET failed with status %d...\n", status);
 
-
     start  = time(NULL);
     length = httpGetLength2(http);
     total  = 0;
@@ -531,5 +600,5 @@ main(int  argc,				/* I - Number of command-line arguments */
 
 
 /*
- * End of "$Id: testhttp.c 5679 2006-06-20 14:45:15Z mike $".
+ * End of "$Id: testhttp.c 10899 2013-03-11 18:44:36Z mike $".
  */
