@@ -1,9 +1,9 @@
 /*
- * "$Id: cert.c 10257 2012-02-12 04:34:46Z mike $"
+ * "$Id: cert.c 10776 2012-12-17 22:17:08Z mike $"
  *
  *   Authentication certificate routines for the CUPS scheduler.
  *
- *   Copyright 2007-2011 by Apple Inc.
+ *   Copyright 2007-2012 by Apple Inc.
  *   Copyright 1997-2006 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -42,7 +42,7 @@
 void
 cupsdAddCert(int        pid,		/* I - Process ID */
              const char *username,	/* I - Username */
-             void       *ccache)	/* I - Kerberos credentials or NULL */
+             int        type)		/* I - AuthType for username */
 {
   int		i;			/* Looping var */
   cupsd_cert_t	*cert;			/* Current certificate */
@@ -66,7 +66,8 @@ cupsdAddCert(int        pid,		/* I - Process ID */
   * Fill in the certificate information...
   */
 
-  cert->pid = pid;
+  cert->pid  = pid;
+  cert->type = type;
   strlcpy(cert->username, username, sizeof(cert->username));
 
   for (i = 0; i < 32; i ++)
@@ -259,16 +260,6 @@ cupsdAddCert(int        pid,		/* I - Process ID */
   write(fd, cert->certificate, strlen(cert->certificate));
   close(fd);
 
- /*
-  * Add Kerberos credentials as needed...
-  */
-
-#ifdef HAVE_GSSAPI
-  cert->ccache = (krb5_ccache)ccache;
-#else
-  (void)ccache;
-#endif /* HAVE_GSSAPI */
-
  /*
   * Insert the certificate at the front of the list...
   */
@@ -308,15 +299,6 @@ cupsdDeleteCert(int pid)		/* I - Process ID */
       else
         prev->next = cert->next;
 
-#ifdef HAVE_GSSAPI
-     /*
-      * Release Kerberos credentials as needed...
-      */
-
-      if (cert->ccache)
-	krb5_cc_destroy(KerberosContext, cert->ccache);
-#endif /* HAVE_GSSAPI */
-
       free(cert);
 
      /*
@@ -449,10 +431,10 @@ cupsdInitCerts(void)
   */
 
   if (!RunUser)
-    cupsdAddCert(0, "root", NULL);
+    cupsdAddCert(0, "root", cupsdDefaultAuthType());
 }
 
 
 /*
- * End of "$Id: cert.c 10257 2012-02-12 04:34:46Z mike $".
+ * End of "$Id: cert.c 10776 2012-12-17 22:17:08Z mike $".
  */
