@@ -1,7 +1,7 @@
 /*
- * "$Id: select.c 9157 2010-06-16 05:27:41Z mike $"
+ * "$Id: select.c 10142 2011-12-02 20:02:15Z mike $"
  *
- *   Select abstraction functions for the Common UNIX Printing System (CUPS).
+ *   Select abstraction functions for the CUPS scheduler.
  *
  *   Copyright 2007-2010 by Apple Inc.
  *   Copyright 2006-2007 by Easy Software Products.
@@ -33,12 +33,12 @@
 
 #ifdef HAVE_EPOLL
 #  include <sys/epoll.h>
-#  include <sys/poll.h>
+#  include <poll.h>
 #elif defined(HAVE_KQUEUE)
 #  include <sys/event.h>
 #  include <sys/time.h>
 #elif defined(HAVE_POLL)
-#  include <sys/poll.h>
+#  include <poll.h>
 #elif defined(__hpux)
 #  include <sys/time.h>
 #else
@@ -947,5 +947,5 @@ find_fd(int fd)				/* I - File descriptor */
 
 
 /*
- * End of "$Id: select.c 9157 2010-06-16 05:27:41Z mike $".
+ * End of "$Id: select.c 10142 2011-12-02 20:02:15Z mike $".
  */
